@@ -19,25 +19,27 @@ import time
 import os
 
 from functools import partial
+from functools import reduce
 
-from ansible.module_utils.six.moves import reduce
 from ansible.parsing.ajson import AnsibleJSONEncoder
 from ansible.plugins.callback import CallbackBase
 
 DOCUMENTATION = '''
     callback: json
-    short_description: Ansible screen output as JSON file
+    short_description: Log Ansible results on filesystem
     version_added: "1.0"
     description:
         - This callback converts all events into a JSON file
-          stored in /var/log/validations
-    type: stdout
+          stored in the selected validations logging directory,
+          as defined by the $VALIDATIONS_LOG_DIR env variable,
+          or the $HOME/validations by default.
+    type: aggregate
     requirements: None
 '''
 
-VALIDATIONS_LOG_DIR = (os.getenv('VALIDATIONS_LOG_DIR')
-                       if os.getenv('VALIDATIONS_LOG_DIR') else
-                       "/var/log/validations")
+VALIDATIONS_LOG_DIR = os.environ.get(
+    'VALIDATIONS_LOG_DIR',
+    os.environ.get('HOME'))
 
 
 def current_time():
@@ -56,15 +58,16 @@ def secondsToStr(t):
 
 class CallbackModule(CallbackBase):
     CALLBACK_VERSION = 2.0
-    CALLBACK_TYPE = 'stdout'
+    CALLBACK_TYPE = 'aggregate'
     CALLBACK_NAME = 'validation_json'
+    CALLBACK_NEEDS_WHITELIST = True
 
     def __init__(self, display=None):
         super(CallbackModule, self).__init__(display)
         self.results = []
         self.simple_results = []
         self.env = {}
-        self.t0 = None
+        self.start_time = None
         self.current_time = current_time()
 
     def _new_play(self, play):
@@ -111,7 +114,7 @@ class CallbackModule(CallbackBase):
         }
 
     def v2_playbook_on_start(self, playbook):
-        self.t0 = time.time()
+        self.start_time = time.time()
         pl = playbook._file_name
         validation_id = os.path.splitext(os.path.basename(pl))[0]
         self.env = {
@@ -171,15 +174,16 @@ class CallbackModule(CallbackBase):
             self.simple_results.append(self._val_task(task.name))
             self.simple_results[-1]['task']['status'] = "FAILED"
             self.simple_results[-1]['task']['hosts'][host.name] = task_result
-        if 'warnings' in task_result.keys():
+        if 'warnings' in task_result.keys() and task_result.get('warnings'):
             self.simple_results.append(self._val_task(task.name))
             self.simple_results[-1]['task']['status'] = "WARNING"
             self.simple_results[-1]['task']['hosts'][host.name] = task_result
 
         end_time = current_time()
-        time_elapsed = secondsToStr(time.time() - self.t0)
+        time_elapsed = secondsToStr(time.time() - self.start_time)
         for result in self.results:
-            result['tasks'][-1]['task']['duration']['end'] = end_time
+            if len(result['tasks']) > 1:
+                result['tasks'][-1]['task']['duration']['end'] = end_time
             result['play']['duration']['end'] = end_time
             result['play']['duration']['time_elapsed'] = time_elapsed
 
